@@ -11,12 +11,13 @@
 //! | Feature | Description | Extra dependencies | Default |
 //! | ------- | ----------- | ------------------ | ------- |
 //! | `config` | Enable support for [config](https://crates.io/crates/config) crate | `config`, `serde/derive` | yes |
+//! | `rt_tokio_1` | Enable support for [tokio](https://crates.io/crates/tokio) crate | `deadpool/rt_tokio_1`, `redis/tokio-comp` | yes |
+//! | `rt_async-std_1` | Enable support for [async-std](https://crates.io/crates/config) crate | `deadpool/rt_async-std_1`, `redis/async-std-comp` | no |
 //!
 //! ## Example
 //!
 //! ```rust,ignore
-//! use deadpool_redis::{cmd, Config};
-//! use deadpool_redis::redis::FromRedisValue;
+//! use deadpool_redis::redis::{cmd, Config, FromRedisValue};
 //!
 //! #[tokio::main]
 //! async fn main() {
@@ -27,7 +28,7 @@
 //!         let mut conn = pool.get().await.unwrap();
 //!         cmd("SET")
 //!             .arg(&["deadpool/test_key", "42"])
-//!             .execute_async(&mut conn)
+//!             .query_async::<_, ()>(&mut conn)
 //!             .await.unwrap();
 //!     }
 //!     {
@@ -44,8 +45,7 @@
 //! ## Example with `config` and `dotenv` crate
 //!
 //! ```rust
-//! use deadpool_redis::cmd;
-//! use deadpool_redis::redis::FromRedisValue;
+//! use deadpool_redis::redis::{cmd, FromRedisValue};
 //! use dotenv::dotenv;
 //! use serde::Deserialize;
 //!
@@ -72,7 +72,7 @@
 //!         let mut conn = pool.get().await.unwrap();
 //!         cmd("SET")
 //!             .arg(&["deadpool/test_key", "42"])
-//!             .execute_async(&mut conn)
+//!             .query_async::<_, ()>(&mut conn)
 //!             .await.unwrap();
 //!     }
 //!     {
@@ -96,7 +96,7 @@
 //!
 //!   ```toml
 //!   [dependencies]
-//!   deadpool-redis = { version = "0.7.1", features = ["config"] }
+//!   deadpool-redis = { version = "0.8", features = ["config"] }
 //!   redis = { version = "0.20", default-features = false, features = ["tls"] }
 //!   ```
 //!
@@ -108,26 +108,32 @@
 //! - MIT license ([LICENSE-MIT](LICENSE-MIT) or <http://opensource.org/licenses/MIT>)
 //!
 //! at your option.
-#![warn(missing_docs)]
+#![warn(missing_docs, unreachable_pub)]
 
-use std::{ops::{Deref, DerefMut}, sync::atomic::{AtomicUsize, Ordering}};
+use std::{
+    ops::{Deref, DerefMut},
+    sync::atomic::{AtomicUsize, Ordering},
+};
 
 use async_trait::async_trait;
 /// Re-export deadpool::managed::PoolConfig
 pub use deadpool::managed::PoolConfig;
 use deadpool::managed::RecycleError;
+/// Re-export deadpool::Runtime;
+pub use deadpool::Runtime;
 use redis::{
-    aio::Connection as RedisConnection, Client, IntoConnectionInfo, RedisError, RedisResult,
+    aio::{Connection as RedisConnection, ConnectionLike},
+    Client, IntoConnectionInfo, RedisError, RedisResult,
 };
 
 /// A type alias for using `deadpool::Pool` with `redis`
-pub type Pool = deadpool::managed::Pool<ConnectionWrapper, RedisError>;
+pub type Pool = deadpool::managed::Pool<Manager, ConnectionWrapper>;
 
 /// A type alias for using `deadpool::PoolError` with `redis`
 pub type PoolError = deadpool::managed::PoolError<RedisError>;
 
 /// A type alias for using `deadpool::Object` with `redis`
-pub type Connection = deadpool::managed::Object<ConnectionWrapper, RedisError>;
+pub type Connection = deadpool::managed::Object<Manager>;
 
 type RecycleResult = deadpool::managed::RecycleResult<RedisError>;
 
@@ -136,17 +142,27 @@ pub use redis;
 
 mod config;
 pub use config::Config;
-mod cmd_wrapper;
-pub use cmd_wrapper::{cmd, Cmd};
-mod pipeline_wrapper;
-pub use pipeline_wrapper::{pipe, Pipeline};
 
 /// A wrapper for `redis::Connection`. The `query_async` and `execute_async`
 /// functions of `redis::Cmd` and `redis::Pipeline` consume the connection.
 /// This wrapper makes it possible to replace the internal connection after
 /// executing a query.
 pub struct ConnectionWrapper {
-    conn: RedisConnection,
+    conn: Connection,
+}
+
+impl ConnectionWrapper {
+    /// Take this object from the pool permanently. This reduces the size of
+    /// the pool.
+    pub fn take(this: Self) -> RedisConnection {
+        Connection::take(this.conn)
+    }
+}
+
+impl From<Connection> for ConnectionWrapper {
+    fn from(conn: Connection) -> Self {
+        Self { conn }
+    }
 }
 
 impl Deref for ConnectionWrapper {
@@ -162,6 +178,38 @@ impl DerefMut for ConnectionWrapper {
     }
 }
 
+impl AsRef<redis::aio::Connection> for ConnectionWrapper {
+    fn as_ref(&self) -> &redis::aio::Connection {
+        &self.conn
+    }
+}
+
+impl AsMut<redis::aio::Connection> for ConnectionWrapper {
+    fn as_mut(&mut self) -> &mut redis::aio::Connection {
+        &mut self.conn
+    }
+}
+
+impl ConnectionLike for ConnectionWrapper {
+    fn req_packed_command<'a>(
+        &'a mut self,
+        cmd: &'a redis::Cmd,
+    ) -> redis::RedisFuture<'a, redis::Value> {
+        self.conn.req_packed_command(cmd)
+    }
+    fn req_packed_commands<'a>(
+        &'a mut self,
+        cmd: &'a redis::Pipeline,
+        offset: usize,
+        count: usize,
+    ) -> redis::RedisFuture<'a, Vec<redis::Value>> {
+        self.conn.req_packed_commands(cmd, offset, count)
+    }
+    fn get_db(&self) -> i64 {
+        self.conn.get_db()
+    }
+}
+
 /// The manager for creating and recyling lapin connections
 pub struct Manager {
     client: Client,
@@ -179,21 +227,25 @@ impl Manager {
 }
 
 #[async_trait]
-impl deadpool::managed::Manager<ConnectionWrapper, RedisError> for Manager {
-    async fn create(&self) -> Result<ConnectionWrapper, RedisError> {
+impl deadpool::managed::Manager for Manager {
+    type Type = RedisConnection;
+    type Error = RedisError;
+    async fn create(&self) -> Result<RedisConnection, RedisError> {
         let conn = self.client.get_async_connection().await?;
-        Ok(ConnectionWrapper { conn })
+        Ok(conn)
     }
-    async fn recycle(&self, conn: &mut ConnectionWrapper) -> RecycleResult {
+    async fn recycle(&self, conn: &mut RedisConnection) -> RecycleResult {
         let ping_number = self.ping_number.fetch_add(1, Ordering::Relaxed).to_string();
-        match cmd(&format!("PING {}", ping_number))
-            .query_async::<String>(conn)
+        match redis::cmd(&format!("PING {}", ping_number))
+            .query_async::<_, String>(conn)
             .await
         {
-            Ok(n) => if n == ping_number {
-                Ok(())
-            } else {
-                Err(RecycleError::Message(String::from("Invalid PING response")))
+            Ok(n) => {
+                if n == ping_number {
+                    Ok(())
+                } else {
+                    Err(RecycleError::Message(String::from("Invalid PING response")))
+                }
             }
             Err(e) => Err(e.into()),
         }
